mod additional_rfactor;
#[allow(warnings)]
mod rfactor_structs;

use std::{fs::File, path::Path};

use additional_rfactor::*;
use anyhow::Result;
use memmap2::Mmap;
use rfactor_structs::TelemInfoV01;

const RFACTOR_SHM_FILE: &str = "/dev/shm";

pub struct TelemetryReader {
    _telemetry_file: File,
    shm: Mmap,
}

impl TelemetryReader {
    pub fn new() -> Result<Self> {
        let file = File::open(Path::new(RFACTOR_SHM_FILE).join(MM_TELEMETRY_FILE_NAME))?;
        let mmap = unsafe { Mmap::map(&file)? };

        Ok(Self {
            _telemetry_file: file,
            shm: mmap,
        })
    }

    pub fn query_telemetry(&self) -> Result<Vec<TelemInfoV01>> {
        let telemetry = rF2Telemetry::try_from(&self.shm[0..rF2Telemetry::SIZE])?;
        let vehicles = telemetry.vehicles();

        Ok(vehicles.to_vec())
    }
}
